@@ -11,7 +11,7 @@ use List::Util qw( first );
 use List::MoreUtils qw( any all uniq first_index );
 use Scalar::Util 'weaken', 'blessed';
 
-our $VERSION   = '0.82';
+our $VERSION   = '0.94';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -68,6 +68,32 @@ sub initialize {
             );
 }
 
+sub reinitialize {
+    my $self = shift;
+    my $pkg  = shift;
+
+    my $meta = blessed $pkg ? $pkg : Class::MOP::class_of($pkg);
+
+    my %existing_classes;
+    if ($meta) {
+        %existing_classes = map { $_ => $meta->$_() } qw(
+            attribute_metaclass
+            method_metaclass
+            wrapped_method_metaclass
+            instance_metaclass
+            constructor_class
+            destructor_class
+            error_class
+        );
+    }
+
+    return $self->SUPER::reinitialize(
+        $pkg,
+        %existing_classes,
+        @_,
+    );
+}
+
 sub _immutable_options {
     my ( $self, @args ) = @_;
 
@@ -227,28 +253,13 @@ sub new_object {
     return $self;
 }
 
-sub _construct_instance {
-    my $class = shift;
-    my $params = @_ == 1 ? $_[0] : {@_};
-    my $meta_instance = $class->get_meta_instance;
-    # FIXME:
-    # the code below is almost certainly incorrect
-    # but this is foreign inheritance, so we might
-    # have to kludge it in the end.
-    my $instance = $params->{'__INSTANCE__'} || $meta_instance->create_instance();
-    foreach my $attr ($class->get_all_attributes()) {
-        $attr->initialize_instance_slot($meta_instance, $instance, $params);
-    }
-    return $instance;
-}
-
 sub superclasses {
     my $self = shift;
     my @supers = @_;
     foreach my $super (@supers) {
         Class::MOP::load_class($super);
         my $meta = Class::MOP::class_of($super);
-        Moose->throw_error("You cannot inherit from a Moose Role ($super)")
+        $self->throw_error("You cannot inherit from a Moose Role ($super)")
             if $meta && $meta->isa('Moose::Meta::Role')
     }
     return $self->SUPER::superclasses(@supers);
@@ -258,11 +269,17 @@ sub superclasses {
 
 sub add_attribute {
     my $self = shift;
-    $self->SUPER::add_attribute(
+    my $attr =
         (blessed $_[0] && $_[0]->isa('Class::MOP::Attribute')
             ? $_[0]
-            : $self->_process_attribute(@_))
-    );
+            : $self->_process_attribute(@_));
+    $self->SUPER::add_attribute($attr);
+    # it may be a Class::MOP::Attribute, theoretically, which doesn't have
+    # 'bare' and doesn't implement this method
+    if ($attr->can('_check_associated_methods')) {
+        $attr->_check_associated_methods;
+    }
+    return $attr;
 }
 
 sub add_override_method_modifier {
@@ -305,29 +322,8 @@ sub _find_next_method_by_name_which_is_not_overridden {
 sub _fix_metaclass_incompatibility {
     my ($self, @superclasses) = @_;
 
-    foreach my $super (@superclasses) {
-        my $meta = Class::MOP::Class->initialize($super);
-
-        my @all_supers = $meta->linearized_isa;
-        shift @all_supers;
-
-        my @super_metas_to_fix = ($meta);
-
-        # We need to check & fix the immediate superclass. If its @ISA
-        # contains a class without a metaclass instance, followed by a
-        # class _with_ a metaclass instance, init a metaclass instance
-        # for classes without one and fix compat up to and including
-        # the class which was already initialized.
-        my $idx = first_index { Class::MOP::class_of($_) } @all_supers;
-
-        push @super_metas_to_fix,
-            map { Class::MOP::Class->initialize($_) } @all_supers[ 0 .. $idx ]
-            if $idx >= 0;
-
-        foreach my $super_meta (@super_metas_to_fix) {
-            $self->_fix_one_incompatible_metaclass($super_meta);
-        }
-    }
+    $self->_fix_one_incompatible_metaclass($_)
+        for map { Moose::Meta::Class->initialize($_) } @superclasses;
 }
 
 sub _fix_one_incompatible_metaclass {
@@ -352,7 +348,7 @@ sub _superclass_meta_is_compatible {
 
     my $super_meta_name
         = $super_meta->is_immutable
-        ? $super_meta->get_mutable_metaclass_name
+        ? $super_meta->_get_mutable_metaclass_name
         : ref($super_meta);
 
     return 1
@@ -376,7 +372,7 @@ sub _reconcile_with_superclass_meta {
 
     my $super_meta_name
         = $super_meta->is_immutable
-        ? $super_meta->get_mutable_metaclass_name
+        ? $super_meta->_get_mutable_metaclass_name
         : ref($super_meta);
 
     my $self_metaclass = ref $self;
@@ -681,8 +677,9 @@ These all default to the appropriate Moose class.
 =item B<< Moose::Meta::Class->create($package_name, %options) >>
 
 This overrides the parent's method in order to accept a C<roles>
-option. This should be an array reference containing one more roles
-that the class does.
+option. This should be an array reference containing roles
+that the class does, each optionally followed by a hashref of options
+(C<-excludes> and C<-alias>).
 
   my $metaclass = Moose::Meta::Class->create( 'New::Class', roles => [...] );
 
@@ -746,10 +743,11 @@ adds it to the class's list of role applications. This I<does not>
 actually apply any role to the class; it is only for tracking role
 applications.
 
-=item B<< $metaclass->does_role($role_name) >>
+=item B<< $metaclass->does_role($role) >>
 
-This returns a boolean indicating whether or not the class does the
-specified role. This tests both the class and its parents.
+This returns a boolean indicating whether or not the class does the specified
+role. The role provided can be either a role name or a L<Moose::Meta::Role>
+object. This tests both the class and its parents.
 
 =item B<< $metaclass->excludes_role($role_name) >>
 
@@ -784,9 +782,7 @@ Throws the error created by C<create_error> using C<raise_error>
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
@@ -794,7 +790,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2009 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
