@@ -6,7 +6,7 @@ use warnings;
 
 use Scalar::Util 'blessed', 'weaken', 'looks_like_number', 'refaddr';
 
-our $VERSION   = '0.82';
+our $VERSION   = '0.94';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Moose::Meta::Method',
@@ -43,11 +43,6 @@ sub new {
     return $self;
 }
 
-# This is here so can_be_inlined can be inherited by MooseX modules.
-sub _expected_constructor_class {
-    return 'Moose::Object';
-}
-
 ## method
 
 sub _initialize_body {
@@ -60,7 +55,9 @@ sub _initialize_body {
     # requires some adaption on the part of
     # the author, after all, nothing is free)
     my $source = 'sub {';
-    $source .= "\n" . 'my $class = shift;';
+    $source .= "\n" . 'my $_instance = shift;';
+
+    $source .= "\n" . 'my $class = Scalar::Util::blessed($_instance) || $_instance;';
 
     $source .= "\n" . 'return $class->Moose::Object::new(@_)';
     $source .= "\n    if \$class ne '" . $self->associated_metaclass->name
@@ -202,7 +199,7 @@ sub _generate_slot_initializer {
 
     if ($is_moose && defined($attr->init_arg) && $attr->is_required && !$attr->has_default && !$attr->has_builder) {
         push @source => ('(exists $params->{\'' . $attr->init_arg . '\'}) ' .
-                        '|| ' . $self->_inline_throw_error('"Attribute (' . $attr->name . ') is required"') .';');
+                        '|| ' . $self->_inline_throw_error('"Attribute (' . quotemeta($attr->name) . ') is required"') .';');
     }
 
     if (($attr->has_default || $attr->has_builder) && !($is_moose && $attr->is_lazy)) {
@@ -329,7 +326,7 @@ sub _generate_type_constraint_check {
     my ($self, $attr, $type_constraint_cv, $type_constraint_obj, $value_name) = @_;
     return (
         $self->_inline_throw_error('"Attribute (' # FIXME add 'dad'
-        . $attr->name
+        . quotemeta( $attr->name )
         . ') does not pass the type constraint because: " . '
         . $type_constraint_obj . '->get_message(' . $value_name . ')')
         . "\n\t unless " .  $type_constraint_cv . '->(' . $value_name . ');'
@@ -375,28 +372,9 @@ L<Class::MOP::Class::Constructor> documentation as well.
 C<Moose::Meta::Method::Constructor> is a subclass of
 L<Moose::Meta::Method> I<and> L<Class::MOP::Method::Constructor>.
 
-=head1 METHODS
+=head1 BUGS
 
-=over 4
-
-=item B<< $metamethod->can_be_inlined >>
-
-This returns true if the method can inlined.
-
-First, it looks at all of the parents of the associated class. If any
-of them have an inlined constructor, then the constructor can be
-inlined.
-
-If none of them have been inlined, it checks to make sure that the
-pre-inlining constructor for the class matches the constructor from
-the expected class.
-
-By default, it expects this constructor come from L<Moose::Object>,
-but subclasses can change this expectation.
-
-If the constructor cannot be inlined it warns that this is the case.
-
-=back
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHORS
 
@@ -404,7 +382,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2009 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
