@@ -7,7 +7,7 @@ use metaclass;
 use Scalar::Util 'blessed';
 use Moose::Util::TypeConstraints ();
 
-our $VERSION   = '0.63';
+our $VERSION   = '0.94';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -20,7 +20,7 @@ __PACKAGE__->meta->add_attribute('role' => (
 sub new {
     my ( $class, %args ) = @_;
 
-    $args{parent} = Moose::Util::TypeConstraints::find_type_constraint('Role');
+    $args{parent} = Moose::Util::TypeConstraints::find_type_constraint('Object');
     my $self      = $class->_new(\%args);
 
     $self->_create_hand_optimized_type_constraint;
@@ -47,10 +47,10 @@ sub parents {
             # if anybody thinks this problematic please discuss on IRC.
             # a possible fix is to add by attr indexing to the type registry to find types of a certain property
             # regardless of their name
-            Moose::Util::TypeConstraints::find_type_constraint($_) 
-                || 
+            Moose::Util::TypeConstraints::find_type_constraint($_)
+                ||
             __PACKAGE__->new( role => $_, name => "__ANON__" )
-        } @{ $self->role->meta->get_roles },
+        } @{ Class::MOP::class_of($self->role)->get_roles },
     );
 }
 
@@ -78,17 +78,17 @@ sub is_subtype_of {
 
     if ( not ref $type_or_name_or_role ) {
         # it might be a role
-        return 1 if $self->role->meta->does_role( $type_or_name_or_role );
+        return 1 if Class::MOP::class_of($self->role)->does_role( $type_or_name_or_role );
     }
 
     my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name_or_role);
 
     return unless defined $type;
-    
+
     if ( $type->isa(__PACKAGE__) ) {
         # if $type_or_name_or_role isn't a role, it might be the TC name of another ::Role type
         # or it could also just be a type object in this branch
-        return $self->role->meta->does_role( $type->role );
+        return Class::MOP::class_of($self->role)->does_role( $type->role );
     } else {
         # the only other thing we are a subtype of is Object
         $self->SUPER::is_subtype_of($type);
@@ -110,39 +110,62 @@ __END__
 
 Moose::Meta::TypeConstraint::Role - Role/TypeConstraint parallel hierarchy
 
+=head1 DESCRIPTION
+
+This class represents type constraints for a role.
+
+=head1 INHERITANCE
+
+C<Moose::Meta::TypeConstraint::Role> is a subclass of
+L<Moose::Meta::TypeConstraint>.
+
 =head1 METHODS
 
 =over 4
 
-=item B<new>
+=item B<< Moose::Meta::TypeConstraint::Role->new(%options) >>
 
-=item B<role>
+This creates a new role type constraint based on the given
+C<%options>.
 
-=item B<hand_optimized_type_constraint>
+It takes the same options as its parent, with two exceptions. First,
+it requires an additional option, C<role>, which is name of the
+constraint's role.  Second, it automatically sets the parent to the
+C<Object> type.
 
-=item B<has_hand_optimized_type_constraint>
+The constructor also overrides the hand optimized type constraint with
+one it creates internally.
 
-=item B<equals>
+=item B<< $constraint->role >>
 
-=item B<is_a_type_of>
+Returns the role name associated with the constraint.
 
-=item B<is_subtype_of>
+=item B<< $constraint->parents >>
 
-=item B<create_child_type>
+Returns all the type's parent types, corresponding to the roles that
+its role does.
 
-=item B<parents>
+=item B<< $constraint->is_subtype_of($type_name_or_object) >>
 
-Return all the parent types, corresponding to the parent classes.
+If the given type is also a role type, then this checks that the
+type's role does the other type's role.
 
-=item B<meta>
+Otherwise it falls back to the implementation in
+L<Moose::Meta::TypeConstraint>.
+
+=item B<< $constraint->create_child_type(%options) >>
+
+This returns a new L<Moose::Meta::TypeConstraint> object with the type
+as its parent.
+
+Note that it does I<not> return a C<Moose::Meta::TypeConstraint::Role>
+object!
 
 =back
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no 
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
@@ -150,7 +173,7 @@ Yuval Kogman E<lt>nothingmuch@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
